package com.keystar.plane.inspection.controller;

import com.drew.imaging.jpeg.JpegProcessingException;
import com.keystar.plane.inspection.bo.NewFileBo;
import com.keystar.plane.inspection.bo.PvNameBo;
import com.keystar.plane.inspection.vo.PhotoResultVo;
import com.keystar.plane.inspection.service.MqttService;
import com.keystar.plane.inspection.service.PhotoService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.service.TangyangDataService;
import com.keystar.plane.inspection.utils.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

@Slf4j
@RestController
@RequestMapping(value = "/photo")
public class PhotoController {

    @Autowired
    PhotoService photoService;

    @Autowired
    PlanePhotoService planePhotoService;

    @Autowired
    MqttService mqttService;

    @Autowired
    TangyangDataService tangyangDataService;

    /**
     *  获取最新的无人机图片文件
     **/
    @GetMapping("/getNewFile")
    public NewFileBo getNewFile(){
        return FileUtil.getNewFilePath();
    }

    /**
     *  发送最新任务图片Mqtt给图像识别
     **/
    @GetMapping("/sendMqtt")
    public void sendMqtt() throws IOException, JpegProcessingException, ParseException {
        mqttService.sendPhotoMqtt();
    }

    /**
     *  根据组串的名字来获取无人机图片和识别结果
     **/
    @PostMapping("/getPhotoMsg")
    public PhotoResultVo getPhotoMsg(@RequestBody PvNameBo pvNameBo){
        return photoService.getPhotoMsg(pvNameBo);
    }


    /**
     *  测试获取小时数
     **/
    @GetMapping("/hour")
    public void getHour(){
        Double monthEquHour = tangyangDataService.getMonthEquHour();
        log.info(monthEquHour.toString());
        Double htsptLose = tangyangDataService.getHtsptLose();
        log.info(htsptLose.toString());
    }

    /**
     *  发送指定目录下的无人机图片给图像识别 + 图片入库
     **/
    @PostMapping("/sendPhoto")
    public List<String> sendPhoto(@RequestBody NewFileBo newFileBo) throws JpegProcessingException, IOException, ParseException {
        File[] files = mqttService.sendPhotoMatt(newFileBo);
        List<String> fileName = new ArrayList<>();
        for (File file : files){
            if (file.isFile()) {
                fileName.add(file.getName());
            }
        }
        return fileName;
    }
}
