package com.keystar.plane.inspection.mqtt;

import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.keystar.mqtt.annotation.MqttHandlerType;
import com.keystar.mqtt.annotation.MqttSubScribe;
import com.keystar.mqtt.handler.AbstractMqttHandler;
import com.keystar.plane.inspection.bo.RecognizeMessageBo;
import com.keystar.plane.inspection.entity.PhotoResultEntity;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.service.PhotoResultService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@MqttHandlerType(
        {
                @MqttSubScribe(topic = "LightSpot_recognize_result", qos = 0)
        }
)
@Slf4j
public class RecognizeMqttHandler implements AbstractMqttHandler {

    private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

    @Autowired
    PhotoResultService photoResultService;

    @Autowired
    PlanePhotoService planePhotoService;

    @Autowired
    PhotoResultMapper photoResultMapper;

    @Override
    public void handle(MqttMessage message) throws IOException {

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddHH:mm:ss");
        String time = simpleDateFormat.format(new Date());
        String messageContent = new String(message.getPayload());
        log.info("======接收到的识别结果mq: " + messageContent);
        JSONObject messageObject = JSONObject.parseObject(messageContent);
        JSONArray dataList = messageObject.getJSONArray("data");

        int missionId = 0;
        int recordId = 0;
        // 将dataList中的结果遍历
        for( Object data : dataList){
            JSONObject dataObject = JSONObject.parseObject(data.toString());
            // 图片id
            String id = dataObject.getString("id");
            // 获取航线id，架次
            if (missionId == 0){
                PlanePhotoEntity photo = new PlanePhotoEntity();
                photo.setId(Long.parseLong(id));
                PlanePhotoEntity byId = planePhotoService.getById(photo);
                missionId = byId.getMissionId();
                recordId = byId.getRecordId();
            }

//            JSONObject id1 = dataObject.getJSONObject("id");
//            String id = id1.toJSONString();
//            String id = dataObject.getJSONObject("id").toJSONString();
            // 检测功能(0热斑检测，1掉串检测)
            List<String> funcIndex = dataObject.getJSONArray("funcIndex").toJavaList(String.class);
            // 可见光图片
            String visiblePath = dataObject.getString("visibleImagePath");
            // 红外图片
            String infraredPath = dataObject.getString("infraredImagePath");
            // 识别结果
            JSONArray detectResult = dataObject.getJSONArray("detectResult");
            List<RecognizeMessageBo> recognizeMessageBoList = detectResult.toJavaList(RecognizeMessageBo.class);
            // 光伏板坐标
            JSONArray arrayBox = dataObject.getJSONArray("arrayBox");
            List<String> arrayBoxList = new ArrayList<>();
            for (int i = 0; i < arrayBox.size(); i++) {
                List<String> list = arrayBox.getJSONArray(i).toJavaList(String.class);
                String join = String.join(",", list);
                arrayBoxList.add(join);
            }
            String join = String.join(";", arrayBoxList);
            // 将光伏板坐标入库
            PlanePhotoEntity photoEntity = new PlanePhotoEntity();
            photoEntity.setId(Long.parseLong(id));
            photoEntity.setArrayBox(join);
            planePhotoService.updateById(photoEntity);

            // 将识别结果入库
            for (RecognizeMessageBo recognizeMessageBo : recognizeMessageBoList){
                PhotoResultEntity photoResultEntity = new PhotoResultEntity();
                photoResultEntity.setId(IdUtil.getSnowflake().nextId());
                photoResultEntity.setPhotoId(Long.parseLong(id));
                photoResultEntity.setArrayIndex(recognizeMessageBo.getArrayIndex());
                photoResultEntity.setRowIndex(recognizeMessageBo.getRowIndex());
                photoResultEntity.setColumnIndex(recognizeMessageBo.getColumnIndex());
                photoResultEntity.setType(recognizeMessageBo.getResult().getType());
                String joined = String.join(",", recognizeMessageBo.getResult().getCoordinates());
                photoResultEntity.setCoordinates(joined);
                photoResultEntity.setTime(time);
                log.info("======识别结果入库：" + photoResultEntity);
                photoResultService.save(photoResultEntity);
            }
        }

        // 将识别返回的异常光伏板数量存入redis
        List<PhotoResultEntity> list = photoResultMapper.queryFltPVPanel(recordId, null);
        String key = "PlaneInspection:abnormalPvPanels:" + missionId;
        RedisUtils.set(key, String.valueOf(list.size()));
    }
}
