package com.keystar.plane.inspection.service.impl;

import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.keystar.plane.inspection.bo.*;
import com.keystar.plane.inspection.constant.LocationConstant;
import com.keystar.plane.inspection.entity.PhotoResultEntity;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.mapper.PlanePhotoMapper;
import com.keystar.plane.inspection.service.PhotoResultService;
import com.keystar.plane.inspection.service.PhotoService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.utils.GisCheckUtils;
import com.keystar.plane.inspection.utils.PhotoUtils;
import com.keystar.plane.inspection.vo.PhotoMessageVo;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.*;
import java.util.*;

@Service
public class PhotoResultServiceImpl extends ServiceImpl<PhotoResultMapper, PhotoResultEntity> implements PhotoResultService {

    @Resource
    PhotoResultMapper photoResultMapper;

    @Resource
    PlanePhotoMapper planePhotoMapper;

    @Autowired
    private MinioService minioService;

    @Autowired
    private PlanePhotoService planePhotoService;

    @Autowired
    private PhotoService photoService;

    @Override
    public PhotoBo queryResultByPhotoId(PhotoBo photoBo) {
        Long id = photoBo.getId();
        List<PhotoResultEntity> photoResultEntities = photoResultMapper.queryResultByPhotoId(id);
        photoBo.setResults(photoResultEntities);
        return photoBo;
    }

    @Override
    public String pic2Base64FromMinio(String minioAddr) {

        String[] paths = StrUtil.splitToArray(minioAddr, "/");

        InputStream inputStream = null;
        ByteArrayOutputStream byteArrayOutputStream = null;

        try {
            inputStream = minioService.getObject(paths[1], StrUtil.sub(minioAddr,
                    CharSequenceUtil.length(paths[1]) + 2, CharSequenceUtil.length(minioAddr)));

            byteArrayOutputStream = new ByteArrayOutputStream();

//            // 将图片按大小，质量压缩
//            Thumbnails.of(inputStream)
//                    .scale(1f) //图片大小（长宽）压缩比例 从0-1，1表示原图
//                    .outputQuality(0.5f)//图片质量压缩比例 从0-1，越接近1质量越好
//                    .toOutputStream(byteArrayOutputStream);

            byte[] bytes = new byte[10240];
            int mReadLength = 0;

            while ((mReadLength = inputStream.read(bytes)) != -1) {
                byteArrayOutputStream.write(bytes, 0, mReadLength);
            }
            // return "data:image/jpeg;base64," + Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
            return Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
        }catch (Exception e) {

        }finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                }catch (Exception e) {

                }
            }
            if (byteArrayOutputStream != null) {
                try {
                    byteArrayOutputStream.close();
                }catch (Exception e) {

                }
            }
        }

        return "";
    }

    @Override
    public String pic2Base64FromPath(String filePath) {

        FileInputStream fileInputStream = null;
        ByteArrayOutputStream byteArrayOutputStream = null;

        try {

            fileInputStream = new FileInputStream(filePath);

            byteArrayOutputStream = new ByteArrayOutputStream();

            byte[] bytes = new byte[10240];
            int mReadLength = 0;

            while ((mReadLength = fileInputStream.read(bytes)) != -1) {
                byteArrayOutputStream.write(bytes, 0, mReadLength);
            }

            return Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
        }catch (Exception e) {

        }finally {
            if (fileInputStream != null) {
                try {
                    fileInputStream.close();
                }catch (Exception e) {

                }
            }
            if (byteArrayOutputStream != null) {
                try {
                    byteArrayOutputStream.close();
                }catch (Exception e) {

                }
            }
        }

        return "";

    }

    @Override
    public List<PlaneInspectionResultBo> listInspectionResult() throws IOException {
        //查库组装巡检结果集合传入函数即可
        List<PlaneInspectionResultBo> planeInspectionResultBoList = new ArrayList<>();

        //查询有效航线中的最新架次
        List<PlanePhotoEntity> photoEntityList = planePhotoService.queryNewRecordByMission();
        log.debug("*******************photoEntityList Size: " + photoEntityList.size());
        int detailNum = 0;
        for (PlanePhotoEntity photoEntity : photoEntityList){

            // 根据架次获取图片和异常信息
            List<PlanePhotoEntity> photoList = planePhotoMapper.queryPhotoByRecordId(photoEntity.getRecordId());
            log.debug("*******************photoList Size: " + photoList.size());
            for (PlanePhotoEntity photo : photoList){
                PlaneInspectionResultBo planeInspectionResultBo = new PlaneInspectionResultBo();
                planeInspectionResultBo.setPics(new ArrayList<>());
                planeInspectionResultBo.setDetails(new ArrayList<>());

                int detailDioe = 0, detailHtspte = 0, detailOce = 0;
                planeInspectionResultBo.setId(String.valueOf(photo.getId()));
                PlaneInspectionPicBo planeInspectionPicBo = new PlaneInspectionPicBo();
//                planeInspectionPicBo.setVisible(pic2Base64FromPath("F:\\photo\\" + photo.getVisibleImagePath()));
                planeInspectionPicBo.setInfraredId(photo.getId() + "1");
                // 将图片画框处理，将图片压缩
                planeInspectionPicBo.setInfrared(photoService.addBorderToBase64(photo));
//                planeInspectionPicBo.setInfrared(pic2Base64FromMinio(photo.getInfraredImagePath()));
                planeInspectionPicBo.setVisibleId(photo.getId() + "2");
                planeInspectionPicBo.setVisible("");
                planeInspectionResultBo.getPics().add(planeInspectionPicBo);

                // 根据图片id获取识别异常信息
                List<PhotoResultEntity> photoResultEntities = photoResultMapper.queryResultByPhotoId(photo.getId());
                if (photoResultEntities.size() != 0){
                    for (PhotoResultEntity photoResultEntity : photoResultEntities){
                        switch (photoResultEntity.getType()){
                            case "diodeFault":
                                detailDioe = 1;
                                break;
                            case "covered":
                                detailHtspte = 1;
                            default:

                        }
                    }
                }
                PlaneInspectionResultDetailBo planeInspectionResultDetailBo = new PlaneInspectionResultDetailBo();
                planeInspectionResultDetailBo.setNum(String.valueOf(detailNum++));
//                // 取最接近图片中心的组串作为名字
//                PhotoMessageVo photoMessage = PhotoUtils.getPhotoMessage( );
//                String[]  pvNameArray = photo.getPvName().split(",");
//                ArrayList<String> arrayList = new ArrayList<>(pvNameArray.length);
//                Collections.addAll(arrayList,pvNameArray);
//                String name = "";
//                double distance = 100.0;
//                for (String pvName : arrayList){
//                    Map<String, List<Location>> pz = LocationConstant.PVLocation.get(pvName.split("HL")[0]);
//                    Location centerPoint = GisCheckUtils.getCenterPoint(pz.get(pvName));
//                    // 判断那个组串中心点坐标距离图片中心点最近
////                    double d = GisCheckUtils.GetDistance(centerPoint, location);
//
//                }

                planeInspectionResultDetailBo.setName(photo.getCenterPvName());
                planeInspectionResultDetailBo.setDioe(String.valueOf(detailDioe));
                planeInspectionResultDetailBo.setHtspte(String.valueOf(detailHtspte));
                planeInspectionResultDetailBo.setOce(String.valueOf(detailOce));
                planeInspectionResultBo.getDetails().add(planeInspectionResultDetailBo);

                planeInspectionResultBoList.add(planeInspectionResultBo);
            }
        }

        return planeInspectionResultBoList;
    }
}
