package com.keystar.plane.inspection.service.impl;

import com.drew.imaging.jpeg.JpegProcessingException;
import com.keystar.plane.inspection.bo.Location;
import com.keystar.plane.inspection.bo.PhotoBo;
import com.keystar.plane.inspection.bo.PvNameBo;
import com.keystar.plane.inspection.constant.LocationConstant;
import com.keystar.plane.inspection.service.PhotoResultService;
import com.keystar.plane.inspection.vo.PhotoMessageVo;
import com.keystar.plane.inspection.vo.PhotoResultVo;
import com.keystar.plane.inspection.vo.PlanePhotoVo;
import com.keystar.plane.inspection.entity.PhotoResultEntity;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.mapper.PlanePhotoMapper;
import com.keystar.plane.inspection.service.PhotoService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.utils.GisCheckUtils;
import com.keystar.plane.inspection.utils.PhotoUtils;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.imageio.stream.ImageOutputStream;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.*;


@Slf4j
@Service
public class PhotoServiceImpl implements PhotoService {

//    @Autowired
//    MqttAsyncClient mqttAsyncClient;

//    private static final ObjectMapper OBJECT_MAPPER = new ObjectMapper();

    @Resource
    PlanePhotoMapper planePhotoMapper;

    @Resource
    PhotoResultMapper photoResultMapper;

    @Autowired
    PlanePhotoService planePhotoService;

    @Autowired
    PhotoResultService photoResultService;


    @Override
    public PlanePhotoVo queryPhoto(String filePath) throws IOException, JpegProcessingException {
        PlanePhotoVo planePhotoVo = new PlanePhotoVo();
//            file = ResourceUtils.getFile("classpath:1114_THRM.JPG");
        File file = new File(filePath);

        // 获取图片中的信息
        PhotoMessageVo photoMessage = PhotoUtils.getPhotoMessage(file.getPath());
        // 根据图片信息，获取图片四个角的经纬度，判断图片在哪个光伏区
        List<Location> degrees = PhotoUtils.getDegrees(photoMessage);
        Location location = new Location();
        location.setLat(photoMessage.getLatitude());
        location.setLon(photoMessage.getLongitude());
        for ( Map.Entry<String, List<Location>> pzMap : LocationConstant.PZLocation.entrySet()){
            List<Location> value = pzMap.getValue();
            // 判断第一个多边形是否全部在第二个多边形内
//            if (GisCheckUtils.isPolygonAllInPolygon(degrees, value)){
//                planePhotoVo.setPZName(pzMap.getKey());
//            }

            // 判断第一个多边形是否存在一个点在第二个多边形内
//            if (GisCheckUtils.isPolygonInPolygon(degrees, value)){
//                planePhotoVo.setPZName(pzMap.getKey());
//            }

            // 当图片与光伏区的重合面积大于图片本身面积的百分之60才判断图片属于这个光伏区
            double photoArea = GisCheckUtils.planarPolygonAreaMeters(degrees);
            double pzArea = GisCheckUtils.CoincidentArea(degrees, value);
            if (pzArea >= photoArea * 0.6){
                planePhotoVo.setPZName(pzMap.getKey());
                log.info("====判断出来无人机图片属于的光伏区是：" + pzMap.getKey());
                log.info("====光伏区与图片的重合面积是：" + pzArea);
            }
        }

        // 查出图片中包含哪些光伏板
        if (planePhotoVo.getPZName() != null){
            String pzName = planePhotoVo.getPZName();
            Map<String, List<Location>> stringListMap = LocationConstant.PVLocation.get(pzName.split("FZ")[1]);
            List<String> pvNameList = new ArrayList<>();

            String centerPvName = "";
            double distance = 50.0;
            for (Map.Entry<String, List<Location>> pvMap : stringListMap.entrySet()){
                // 判断第一个多边形是否存在一个点在第二个多边形内
//                if (GisCheckUtils.isPolygonInPolygon(pvMap.getValue(), degrees)){
//                    pvNameList.add(pvMap.getKey());
//                }
                // 判断第一个多边形是否全部在第二个多边形内
//                if (GisCheckUtils.isPolygonAllInPolygon(pvMap.getValue(), degrees)){
//                    pvNameList.add(pvMap.getKey());
//                }

                // 当图片与的支架重合面积大于支架本身面积的百分之60才判断这个支架属于这个图片
                double pvArea = GisCheckUtils.planarPolygonAreaMeters(pvMap.getValue());
                double coincidentArea = GisCheckUtils.CoincidentArea(degrees, pvMap.getValue());
                if (coincidentArea >= pvArea * 0.6){
                    pvNameList.add(pvMap.getKey());
                    log.info("====判断出来无人机图片包涵的组串是：" + pvMap.getKey());
                    log.info("====组串与图片的重合面积是：" + coincidentArea);
                }

                // 获取组串中心点的坐标
                Location centerPoint = GisCheckUtils.getCenterPoint(pvMap.getValue());
                // 判断那个组串中心点坐标距离图片中心点最近
                double d = GisCheckUtils.GetDistance(centerPoint, location);
                if (d < distance){
                    distance = d;
                    centerPvName = pvMap.getKey();
                }
            }
            log.info("=====组串的名字是：" + centerPvName  );
            log.info("=====与图片中心点相差的距离是：" + distance);
            planePhotoVo.setCenterPvName(centerPvName);
            planePhotoVo.setPVNames(pvNameList);
        }

        return planePhotoVo;
    }

    /**
     *  通过组串名字获取对应的图片和识别结果
     **/
    @Override
    public PhotoResultVo getPhotoMsg(PvNameBo pvNameBo) {
        // 获取这个组串最新的图片
        List<PhotoBo> photoBoList = planePhotoService.queryPhotoByPvName(pvNameBo);
        // 获取这个组串的图片总数
        PvNameBo nameBo = new PvNameBo();
        nameBo.setPvName(pvNameBo.getPvName());
        List<PlanePhotoEntity> photoEntityList = planePhotoMapper.getPhotoBo(nameBo);
        PhotoResultVo photoResultVo = new PhotoResultVo();
        photoResultVo.setTotal(photoEntityList.size());
        // 根据图片id查询识别结果
        List<PhotoBo> photoBos = new ArrayList<>();
        for (PhotoBo photoBo : photoBoList){
            Long id = photoBo.getId();
            List<PhotoResultEntity> photoResultEntities = photoResultMapper.queryResultByPhotoId(id);
            photoBo.setResults(photoResultEntities);
            photoBos.add(photoBo);
        }
        photoResultVo.setData(photoBos);
        return photoResultVo;
    }

    @Override
    public PlanePhotoVo queryPhoto(PhotoMessageVo photoMessage, Location location) throws IOException, JpegProcessingException {
        PlanePhotoVo planePhotoVo = new PlanePhotoVo();
        // 根据图片信息，获取图片四个角的经纬度，判断图片在哪个光伏区
        List<Location> degrees = PhotoUtils.getDegrees(photoMessage);
        for ( Map.Entry<String, List<Location>> pzMap : LocationConstant.PZLocation.entrySet()){
            List<Location> value = pzMap.getValue();

            // 当图片与光伏区的重合面积大于图片本身面积的百分之60才判断图片属于这个光伏区
            double photoArea = GisCheckUtils.planarPolygonAreaMeters(degrees);
            double pzArea = GisCheckUtils.CoincidentArea(degrees, value);
            if (pzArea >= photoArea * 0.6){
                planePhotoVo.setPZName(pzMap.getKey());
                log.info("====判断出来无人机图片属于的光伏区是：" + pzMap.getKey());
                log.info("====光伏区与图片的重合面积是：" + pzArea);
            }
        }

        // 判断坐标点在那个组串上
        if (planePhotoVo.getPZName() != null){
            String pzName = planePhotoVo.getPZName();
            Map<String, List<Location>> stringListMap = LocationConstant.PVLocation.get(pzName.split("FZ")[1]);
            List<String> pvNameList = new ArrayList<>();
            for (Map.Entry<String, List<Location>> pvMap : stringListMap.entrySet()){
                if (GisCheckUtils.isPointInPolygon(location, pvMap.getValue())){
                    pvNameList.add(pvMap.getKey());
                }
            }
            planePhotoVo.setPVNames(pvNameList);
        }


        return planePhotoVo;
    }

    /**
     *  将minio图片画框并转成base64
     **/
    @Override
    public String addBorderToBase64(PlanePhotoEntity photo) throws IOException {
        // 获取图片（base64）
        String photoBase64 = photoResultService.pic2Base64FromMinio(photo.getInfraredImagePath());
        // base64 转 BufferedImage
        BufferedImage bufferedImage = PhotoUtils.base64ToBufferedImage(photoBase64);
        // 画框
        String arrayBox = photo.getArrayBox();
        String[] array = arrayBox.split(";");
        ArrayList<String> arrayList = new ArrayList<>(array.length);
        Collections.addAll(arrayList,array);
        for (String str : arrayList){
            String[] pvArray = str.split(",");
            int width = Integer.parseInt(pvArray[2]) - Integer.parseInt(pvArray[0]);
            int height = Integer.parseInt(pvArray[3]) - Integer.parseInt(pvArray[1]);
            bufferedImage = PhotoUtils.addBorders(bufferedImage, Integer.parseInt(pvArray[0]), Integer.parseInt(pvArray[1]), width, height);
        }
        // BufferedImage 转 inputStream
        ByteArrayOutputStream bs = new ByteArrayOutputStream();
        ImageOutputStream imOut = ImageIO.createImageOutputStream(bs);
        ImageIO.write(bufferedImage, "jpg", imOut);
        InputStream inputStream = new ByteArrayInputStream(bs.toByteArray());
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        // 然后将图片进行压缩再转成base64
        Thumbnails.of(inputStream)
                .scale(1f) //图片大小（长宽）压缩比例 从0-1，1表示原图
                .outputQuality(0.5f)//图片质量压缩比例 从0-1，越接近1质量越好
                .toOutputStream(byteArrayOutputStream);

        return Base64.getEncoder().encodeToString(byteArrayOutputStream.toByteArray());
    }


}
