package com.keystar.plane.inspection.service.impl;

/**
 * @Author Lucien
 * @Date 2023-03-02 08:00:00
 * @Describe 报告接口实现类
 */

import cn.hutool.core.util.ArrayUtil;
import com.keystar.plane.inspection.bo.PlaneInspectionResultBo;
import com.keystar.plane.inspection.config.MinIoConfig;
import com.keystar.plane.inspection.constant.MissionConstant;
import com.keystar.plane.inspection.entity.PlanePhotoEntity;
import com.keystar.plane.inspection.mapper.PhotoResultMapper;
import com.keystar.plane.inspection.service.IReportService;
import com.keystar.plane.inspection.service.PhotoResultService;
import com.keystar.plane.inspection.service.PlanePhotoService;
import com.keystar.plane.inspection.service.TangyangDataService;
import com.keystar.plane.inspection.vo.UploadVo;
import freemarker.cache.ClassTemplateLoader;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.Version;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;

import org.apache.commons.fileupload.FileItem;

import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.http.MediaType;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.annotation.Resource;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Slf4j
@RefreshScope
//@EnableAspectJAutoProxy(exposeProxy = true)
public class ReportServiceImpl implements IReportService {

    final MinioService minioService;

    @Autowired
    private PlanePhotoService planePhotoService;

    @Resource
    private PhotoResultMapper photoResultMapper;

    @Autowired
    private TangyangDataService tangyangDataService;

    @Autowired
    private PhotoResultService photoResultService;

    @Value("${report.bucket}")
    private String bucket;

    @Autowired
    public ReportServiceImpl(MinioService minioService) {
        this.minioService = minioService;
    }

    //具体参数参照报告关键词
    @Override
    public void createReport(Map<String, String> mFlyCountMap, Map<String, String> mSummaryMap, List<PlaneInspectionResultBo> planeInspectionResultBoList) {

        FileOutputStream fileOutputStream = null;
        OutputStreamWriter outputStreamWriter = null;
        File mFile = null;

        try {

            Calendar mCalendar = Calendar.getInstance();
            mCalendar.setTime(new Date());

            Map<String, Object> mValueMap = new HashMap<>();
            //报告标题(不用改)
            mValueMap.put("ReportTitle", "大唐安徽新能源唐阳光伏电站无人机智能巡检");
            mCalendar.add(Calendar.MONTH, -1);
            //报告福标题(已适配不用修改, 每个月生成一次, 可以每个月的第一天凌晨生成)
            mValueMap.put("ReportSubTitle", new SimpleDateFormat("yyyy年MM月").format(mCalendar.getTime()) + " 光伏区月度巡检报告");
            mCalendar.setTime(new Date());
            //生成报告日期(已适配不用修改)
            mValueMap.put("GenerateDate", new SimpleDateFormat("yyyy/MM/dd").format(mCalendar.getTime()));

            //电站名字(不需要修改)
            mValueMap.put("StationName", "大唐安徽新能源 唐阳光伏电站");
            //电站占地(问钰尧拿数据后填死即可)
            mValueMap.put("StationArea", "1000 亩");
            //电站装机容量(问钰尧拿数据后填死即可)
            mValueMap.put("StationInstalledCapacity", "50MW");
            //巡检设备(不需要修改)
            mValueMap.put("InspectionUAV", "DJI MK300 H20T");
            //巡检类型(不需要修改)
            mValueMap.put("InspectionType", "可见光、红外");

            mValueMap.putAll(mFlyCountMap);

            mValueMap.putAll(mSummaryMap);

            mValueMap.put("pirs", planeInspectionResultBoList);


            Configuration configuration = new Configuration(new Version("2.3.28"));
            configuration.setDefaultEncoding(StandardCharsets.UTF_8.toString());

            ClassTemplateLoader classTemplateLoader = new ClassTemplateLoader(this.getClass(), "/template/");
            configuration.setTemplateLoader(classTemplateLoader);

            Template template = configuration.getTemplate("report.ftl", StandardCharsets.UTF_8.toString());

            //todo: 记得修改路径和名字(这里后面确定路径后要挂载目录)
            mFile = new File("唐阳_"+ new SimpleDateFormat("yyyy年MM月").format(mCalendar.getTime()) +"_无人机月度巡检报告.doc");

            fileOutputStream = new FileOutputStream(mFile);

            outputStreamWriter = new OutputStreamWriter(fileOutputStream, StandardCharsets.UTF_8);

            template.process(mValueMap, outputStreamWriter);

            outputStreamWriter.flush();

            // 将生成的报告上传到minio上面
            DiskFileItemFactory factory = new DiskFileItemFactory(16, null);
            FileItem fileItem = factory.createItem("file", MediaType.APPLICATION_OCTET_STREAM_VALUE,
                    true, mFile.getName());
            fileItem.getOutputStream();// 不知道为啥代码不会自动初始化outputstream只能多加这一行

            try(InputStream is = new FileInputStream(mFile);
                OutputStream os = fileItem.getOutputStream()){
                IOUtils.copy(is, os);
            }catch (Exception e){
                mFile.delete();
                throw e;
            }

            UploadVo fileVo = new UploadVo();
            fileVo.setFile(new CommonsMultipartFile(fileItem));
            fileVo.setBucketName(bucket);
            SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd");
            fileVo.setResourcePath(simpleDateFormat.format(new Date()));
            minioService.upload(fileVo);
        }catch (Exception e) {
            //生成失败删除垃圾文件
            if (mFile != null) {
                if (mFile.exists()) {
                    mFile.delete();
                }
            }
        }finally {
            if (outputStreamWriter != null) {
                try {
                    outputStreamWriter.close();
                }catch (Exception e) {

                }
            }
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close();
                }catch (Exception e) {

                }
            }
        }

    }

    @Override
    public void getMonthReport() throws ParseException, IOException {
        //((ReportServiceImpl)AopContext.currentProxy()).genMonthReport();
        genMonthReport();
    }

    @Async
    public void genMonthReport() throws ParseException, IOException {

        Map<String, String> mFlyCountMap = new HashMap<>();

        //本月飞行总架次
        mFlyCountMap.put("TotalSortie", planePhotoService.getTotalSortie().toString());
        //有效巡检架次
        mFlyCountMap.put("EffectiveSortie", planePhotoService.getEffectiveSortie().toString());
        //有效巡检天数
        mFlyCountMap.put("EffectiveInspectionDate", planePhotoService.getEffectiveInspectionDate().toString());

        Map<String, String> mSummaryMap = new HashMap<>();
        //方阵总数(16不需要修改)
        mSummaryMap.put("TotalFZ", "16");
        //组串总数(4624不需要修改)
        mSummaryMap.put("TotalPVString", "4624");
        //光伏板总数(124848不需要修改)
        mSummaryMap.put("TotalPVPanel", "124848");
        // 判断无人机任务是否全部覆盖
        List<Integer> effectiveMission = planePhotoService.getEffectiveMission();
        if (effectiveMission.size() == 3){
            //无人机巡检覆盖方阵数
            mSummaryMap.put("CoveredFZ", "16");
            //无人机巡检覆盖方阵百分比
            mSummaryMap.put("CoveredFZPct", "100%");
            //本月巡检覆盖组串总数
            mSummaryMap.put("CoveredPVString", "4624");
            //本月巡检覆盖组串百分比
            mSummaryMap.put("CoveredPVStringPct", "100%");
            //本月巡检覆盖光伏板总数
            mSummaryMap.put("CoveredPVPanel", "124848");
            //本月巡检覆盖光伏板百分比
            mSummaryMap.put("CoveredPVPanelPct", "100%");
            //无人机未巡检方阵数
            mSummaryMap.put("UncoveredFZ", "0");
            //无人机未巡检方阵百分比
            mSummaryMap.put("UnCoveredFZPct", "0%");
            //本月未巡检组串总数
            mSummaryMap.put("UncoveredPVString", "0");
            //本月未巡检组串百分比
            mSummaryMap.put("UncoveredPVStringPct", "0%");
            //本月未巡检光伏板总数
            mSummaryMap.put("UncoveredPVPanel", "0");
            //本月未巡检光伏板百分比
            mSummaryMap.put("UncoveredPVPanelPct", "0%");
        }

//        //查询本月存在异常方阵数
//        Integer FltFZ =  planePhotoService.getFltFZ();
//        mSummaryMap.put("FltFZ", FltFZ.toString());
//        double FltFZPct = (FltFZ.doubleValue() / 16.0) * 100.0;
//        String format = String.format("%.2f", FltFZPct);
//        //存在异常方阵百分比
//        mSummaryMap.put("FltFZPct", format + "%");

        // 获取固定的三条航线下的最新架次id
        List<PlanePhotoEntity> photoEntityList = planePhotoService.queryNewRecordByMission();

        // 二极管故障
        Integer DioFZ = 0,DioPVString = 0,DioPVPanel = 0;
        for (PlanePhotoEntity photoEntity : photoEntityList){
            Integer recordId = photoEntity.getRecordId();
            DioFZ = DioFZ + photoResultMapper.queryFltPz(recordId, "diodeFault").size();
            DioPVString = DioPVString + photoResultMapper.queryFltPVString(recordId, "diodeFault").size();
            DioPVPanel = DioPVPanel + photoResultMapper.queryFltPVPanel(recordId, "diodeFault").size();
        }
        //存在二极管故障的方阵数
        mSummaryMap.put("DioFZ", Integer.toString(DioFZ));
        //存在二极管故障的方阵百分比
        mSummaryMap.put("DioFZPct", String.format("%.2f",(DioFZ.doubleValue() / 16.0) * 100.0) + "%");
        //存在二极管故障的组串总数
        mSummaryMap.put("DioPVString", Integer.toString(DioPVString));
        //存在二极管故障的组串百分比
        mSummaryMap.put("DioPVStringPct", String.format("%.2f",(DioPVString.doubleValue() / 4624.0) * 100.0) + "%");
        //存在二极管故障的光伏板总数
        mSummaryMap.put("DioPVPanel", Integer.toString(DioPVPanel));
        //存在二极管故障的光伏板百分比
        mSummaryMap.put("DioPVPanelPct", String.format("%.2f",(DioPVPanel.doubleValue() / 124848.0) * 100.0) + "%");


        Integer HtsptFZ = 0,HtsptPVString = 0,HtsptPVPanel = 0;
        for (PlanePhotoEntity photoEntity : photoEntityList){
            Integer recordId = photoEntity.getRecordId();
            HtsptFZ = HtsptFZ + photoResultMapper.queryFltPz(recordId, "covered").size();
            HtsptPVString = HtsptPVString + photoResultMapper.queryFltPVString(recordId, "covered").size();
            HtsptPVPanel = HtsptPVPanel + photoResultMapper.queryFltPVPanel(recordId, "covered").size();
        }
        //存在热斑的方阵数
        mSummaryMap.put("HtsptFZ", Integer.toString(HtsptFZ));
        //存在热斑的方阵百分比
        if (HtsptFZ > 16){
            mSummaryMap.put("HtsptFZPct", "100.0" + "%");
        }else {
            mSummaryMap.put("HtsptFZPct", String.format("%.2f", (HtsptFZ.doubleValue() / 16.0) * 100.0) + "%" );
        }
        //存在热斑的组串总数
        mSummaryMap.put("HtsptPVString", Integer.toString(HtsptPVString));
        //存在热斑的组串百分比
        mSummaryMap.put("HtsptPVStringPct", String.format("%.2f", (HtsptPVString.doubleValue() / 4624.0) * 100.0) + "%");
        //存在热斑的光伏板总数
        mSummaryMap.put("HtsptPVPanel", Integer.toString(HtsptPVPanel));
        //存在热斑的光伏板百分比
        mSummaryMap.put("HtsptPVPanelPct", String.format("%.2f", (HtsptPVPanel.doubleValue() / 124848.0) * 100.0) + "%");


        //存在掉串的方阵数
        mSummaryMap.put("OcFZ", "0");
        //存在掉串的方阵百分比
        mSummaryMap.put("OcFZPct", "0%");
        //存在掉串的组串总数
        mSummaryMap.put("OcPVString", "0");
        //存在掉串的组串百分比
        mSummaryMap.put("OcPVStringPct", "0%");
        //存在掉串的光伏板总数
        mSummaryMap.put("OcPVPanel", "0");
        //存在掉串的光伏板百分比
        mSummaryMap.put("OcPVPanelPct", "0%");

//        //存在异常组串总数
//        Integer FltPVString = DioPVString + HtsptPVString;
//        mSummaryMap.put("FltPVString", String.valueOf(FltPVString));
//        //存在异常组串百分比
//        mSummaryMap.put("FltPVStringPct", String.format("%.2f", (FltPVString.doubleValue() / 4624.0) * 100.0) + "%");
//
//        //存在异常光伏板总数
//        Integer FltPVPanel = DioPVPanel + HtsptPVPanel;
//        mSummaryMap.put("FltPVPanel", String.valueOf(FltPVPanel));
//        //存在异常光伏板百分比
//        mSummaryMap.put("FltPVPanelPct", String.format("%.2f", (FltPVPanel.doubleValue() / 124848.0) * 100.0) + "%");


        // 存在异常
        Integer FltFZ = 0, FltPVString = 0, FltPVPanel = 0;
        for (PlanePhotoEntity photoEntity : photoEntityList){
            Integer recordId = photoEntity.getRecordId();
            FltFZ = FltFZ + photoResultMapper.queryFltPz(recordId, null).size();
            FltPVString = FltPVString + photoResultMapper.queryFltPVString(recordId, null).size();
            FltPVPanel = FltPVPanel + photoResultMapper.queryFltPVPanel(recordId, null).size();
        }
        //存在异常方阵数
        mSummaryMap.put("FltFZ", Integer.toString(FltFZ));
        //存在异常方阵百分比
        mSummaryMap.put("FltFZPct", String.format("%.2f", (FltFZ.doubleValue() / 16.0) * 100.0) + "%");
        //存在异常组串总数
        mSummaryMap.put("FltPVString", Integer.toString(FltPVString));
        //存在异常组串百分比
        mSummaryMap.put("FltPVStringPct", String.format("%.2f", (FltPVString.doubleValue() / 4624.0) * 100.0) + "%");
        //存在异常光伏板总数
        mSummaryMap.put("FltPVPanel", Integer.toString(FltPVPanel));
        //存在异常光伏板百分比
        mSummaryMap.put("FltPVPanelPct", String.format("%.2f", (FltPVPanel.doubleValue() / 124848.0) * 100.0) + "%");

        //下面这些可能要宇峰那边帮忙算
        //二极管故障损失量
        Double dioLose = tangyangDataService.getDioLose();
        mSummaryMap.put("DioLose", String.valueOf(dioLose));
        //热斑损失电量
        Double htsptLose = tangyangDataService.getHtsptLose();
        mSummaryMap.put("HtsptLose",String.valueOf(htsptLose));
        //掉串损失电量
        mSummaryMap.put("OcLose", "0");
        //月度损失电量
        Double totalLose = dioLose + htsptLose;
        mSummaryMap.put("TotalLose", String.valueOf(totalLose));

        System.out.println(mFlyCountMap.toString() + mSummaryMap.toString());

        //查库组装巡检结果集合传入函数即可
        List<PlaneInspectionResultBo> planeInspectionResultBoList = photoResultService.listInspectionResult();
//        List<PlaneInspectionResultBo> planeInspectionResultBoList = photoResultService.listInspectionResult();

//        reportService.createReport(mFlyCountMap, mSummaryMap, planeInspectionResultBoList);
        // 多线程
        createReport(mFlyCountMap, mSummaryMap, planeInspectionResultBoList);
    }
}
